@@ -272,7 +272,12 @@ thread_initialization(Goal) :-
     call(Goal),
     !.
 
+%!  '$thread_init'
+%
+%   Called by start_thread() from pl-thread.c before the thread's goal.
+
 '$thread_init' :-
+    set_prolog_flag(toplevel_thread, false),
     (   '$at_thread_initialization'(Goal),
         (   call(Goal)
         ->  fail
@@ -561,13 +566,13 @@ initialise_error(E) :-
 initialise_prolog :-
     '$clean_history',
     apply_defines,
-    apple_setup_app,                            % MacOS cwd/locale setup for swipl-win
     init_optimise,
     '$run_initialization',
     '$load_system_init_file',                   % -F file
     set_toplevel,                               % set `toplevel_goal` flag from -t
     '$set_file_search_paths',                   % handle -p alias=dir[:dir]*
     init_debug_flags,
+    setup_app,
     start_pldoc,                                % handle --pldoc[=port]
     main_thread_init.
 
@@ -583,11 +588,13 @@ main_thread_init :-
     current_prolog_flag(xpce, true),
     exists_source(library(epilog)),
     !,
+    setup_theme,
     use_module(library(epilog)),
     call(epilog([ init(user_thread_init),
                   main(true)
                 ])).
 main_thread_init :-
+    setup_theme,
     user_thread_init.
 
 %!  user_thread_init
@@ -599,6 +606,7 @@ user_thread_init :-
     argv_prolog_files(Files, ScriptMode),
     load_init_file(ScriptMode),                 % -f file
     catch(setup_colors, E, print_message(warning, E)),
+    '$load_history',
     win_associated_files(Files),                % swipl-win: cd and update title
     '$load_script_file',                        % -s file (may be repeated)
     load_associated_files(Files),
@@ -617,6 +625,19 @@ user_thread_init :-
         )
     ).
 
+%!  setup_theme
+
+setup_theme :-
+    current_prolog_flag(theme, Theme),
+    exists_source(library(theme/Theme)),
+    !,
+    use_module(library(theme/Theme)).
+setup_theme.
+
+%!  apply_defines
+%
+%   Handle -Dflag[=value] options
+
 apply_defines :-
     '$cmd_option_val'(defines, Defs),
     apply_defines(Defs).
@@ -641,7 +662,7 @@ apply_define(Def) :-
 	->  true
 	;   Value = Value0
 	),
-	create_prolog_flag(Flag, Value, [warn_not_accessed(true)])
+	set_defined(Flag, Value)
     ).
 apply_define(Def) :-
     atom_concat('no-', Flag, Def),
@@ -658,7 +679,7 @@ set_user_boolean_flag(Flag, Value) :-
     ;   set_prolog_flag(Flag, Value)
     ).
 set_user_boolean_flag(Flag, Value) :-
-    create_prolog_flag(Flag, Value, [warn_not_accessed(true)]).
+    set_defined(Flag, Value).
 
 text_flag_value(integer, Text, Int) :-
     atom_number(Text, Int),
@@ -671,31 +692,21 @@ text_flag_value(term, Text, Term) :-
     !.
 text_flag_value(_, Value, Value).
 
-:- if(current_prolog_flag(apple,true)).
-apple_set_working_directory :-
-    (   expand_file_name('~', [Dir]),
-	exists_directory(Dir)
-    ->  working_directory(_, Dir)
-    ;   true
-    ).
+set_defined(Flag, Value) :-
+    define_options(Flag, Options), !,
+    create_prolog_flag(Flag, Value, Options).
 
-apple_set_locale :-
-    (   getenv('LC_CTYPE', 'UTF-8'),
-	apple_current_locale_identifier(LocaleID),
-	atom_concat(LocaleID, '.UTF-8', Locale),
-	catch(setlocale(ctype, _Old, Locale), _, fail)
-    ->  setenv('LANG', Locale),
-        unsetenv('LC_CTYPE')
-    ;   true
-    ).
+%!  define_options(+Flag, -Options)
+%
+%   Define the options with which to create   Flag. This can be used for
+%   known flags to control -for example- their type.
 
-apple_setup_app :-
-    current_prolog_flag(apple, true),
-    current_prolog_flag(console_menu, true),	% SWI-Prolog.app on MacOS
-    apple_set_working_directory,
-    apple_set_locale.
-:- endif.
-apple_setup_app.
+define_options('SDL_VIDEODRIVER', []).
+define_options(_, [warn_not_accessed(true)]).
+
+%!  init_optimise
+%
+%   Load library(apply_macros) if ``-O`` is effective.
 
 init_optimise :-
     current_prolog_flag(optimise, true),
@@ -878,6 +889,179 @@ load_setup_file(File) :-
                      ]), _, fail).
 
 
+%!  setup_app is det.
+%
+%   When running as an "app", behave as such. The behaviour depends on
+%   the platform.
+%
+%     - Windows
+%       If Prolog is started using --win_app, try to change directory
+%       to <My Documents>\Prolog.
+
+:- if(current_prolog_flag(windows,true)).
+
+setup_app :-
+    current_prolog_flag(associated_file, _),
+    !.
+setup_app :-
+    '$cmd_option_val'(win_app, true),
+    !,
+    catch(my_prolog, E, print_message(warning, E)).
+setup_app.
+
+my_prolog :-
+    win_folder(personal, MyDocs),
+    atom_concat(MyDocs, '/Prolog', PrologDir),
+    (   ensure_dir(PrologDir)
+    ->  working_directory(_, PrologDir)
+    ;   working_directory(_, MyDocs)
+    ).
+
+ensure_dir(Dir) :-
+    exists_directory(Dir),
+    !.
+ensure_dir(Dir) :-
+    catch(make_directory(Dir), E, (print_message(warning, E), fail)).
+
+:- elif(current_prolog_flag(apple, true)).
+use_app_settings(true).                        % Indicate we need app settings
+
+setup_app :-
+    apple_set_locale,
+    current_prolog_flag(associated_file, _),
+    !.
+setup_app :-
+    current_prolog_flag(bundle, true),
+    current_prolog_flag(executable, Exe),
+    file_base_name(Exe, 'SWI-Prolog'),
+    !,
+    setup_macos_app.
+setup_app.
+
+apple_set_locale :-
+    (   getenv('LC_CTYPE', 'UTF-8'),
+        apple_current_locale_identifier(LocaleID),
+        atom_concat(LocaleID, '.UTF-8', Locale),
+        catch(setlocale(ctype, _Old, Locale), _, fail)
+    ->  setenv('LANG', Locale),
+        unsetenv('LC_CTYPE')
+    ;   true
+    ).
+
+setup_macos_app :-
+    restore_working_directory,
+    !.
+setup_macos_app :-
+    expand_file_name('~/Documents/Prolog', [PrologDir]),
+    (   exists_directory(PrologDir)
+    ->  true
+    ;   catch(make_directory(PrologDir), MkDirError,
+              print_message(warning, MkDirError))
+    ),
+    catch(working_directory(_, PrologDir), CdError,
+          print_message(warning, CdError)),
+    !.
+setup_macos_app.
+
+:- elif(current_prolog_flag(emscripten, true)).
+setup_app.
+:- else.
+use_app_settings(true).                        % Indicate we need app settings
+
+% Other (Unix-like) platforms.
+setup_app :-
+    running_as_app,
+    restore_working_directory,
+    !.
+setup_app.
+
+%!  running_as_app is semidet.
+%
+%   True if we were started from the dock.
+
+running_as_app :-
+%   getenv('FLATPAK_SANDBOX_DIR', _),
+    current_prolog_flag(epilog, true),
+    stream_property(In, file_no(0)),
+    \+ stream_property(In, tty(true)),
+    !.
+
+:- endif.
+
+
+:- if((current_predicate(use_app_settings/1),
+       use_app_settings(true))).
+
+
+                /*******************************
+                *    APP WORKING DIRECTORY     *
+                *******************************/
+
+save_working_directory :-
+    working_directory(WD, WD),
+    app_settings(Settings),
+    (   Settings.get(working_directory) == WD
+    ->  true
+    ;   app_save_settings(Settings.put(working_directory, WD))
+    ).
+
+restore_working_directory :-
+    at_halt(save_working_directory),
+    app_settings(Settings),
+    WD = Settings.get(working_directory),
+    catch(working_directory(_, WD), _, fail),
+    !.
+
+                /*******************************
+                *           SETTINGS           *
+                *******************************/
+
+%!  app_settings(-Settings:dict) is det.
+%
+%   Get a dict holding the persistent application settings.
+
+app_settings(Settings) :-
+    app_settings_file(File),
+    access_file(File, read),
+    catch(setup_call_cleanup(
+              open(File, read, In, [encoding(utf8)]),
+              read_term(In, Settings, []),
+              close(In)),
+          Error,
+          (print_message(warning, Error), fail)),
+    !.
+app_settings(#{}).
+
+%!  app_save_settings(+Settings:dict) is det.
+%
+%   Save the given application settings dict.
+
+app_save_settings(Settings) :-
+    app_settings_file(File),
+    catch(setup_call_cleanup(
+              open(File, write, Out, [encoding(utf8)]),
+              write_term(Out, Settings,
+                         [ quoted(true),
+                           module(system), % default operators
+                           fullstop(true),
+                           nl(true)
+                         ]),
+              close(Out)),
+          Error,
+          (print_message(warning, Error), fail)).
+
+
+app_settings_file(File) :-
+    absolute_file_name(user_app_config('app_settings.pl'), File,
+                       [ access(write),
+                         file_errors(fail)
+                       ]).
+:- endif.% app_settings
+
+                /*******************************
+                *           TOPLEVEL           *
+                *******************************/
+
 :- '$hide'('$toplevel'/0).              % avoid in the GUI stacktrace
 
 %!  '$toplevel'
@@ -999,11 +1183,22 @@ prolog :-
 '$query_loop' :-
     break_level(BreakLev),
     setup_call_cleanup(
-        notrace(call_repl_loop_hook(begin, BreakLev)),
+        notrace(call_repl_loop_hook(begin, BreakLev, IsToplevel)),
         '$query_loop'(BreakLev),
-        notrace(call_repl_loop_hook(end, BreakLev))).
+        notrace(call_repl_loop_hook(end, BreakLev, IsToplevel))).
 
-call_repl_loop_hook(BeginEnd, BreakLev) :-
+call_repl_loop_hook(begin, BreakLev, IsToplevel) =>
+    (   current_prolog_flag(toplevel_thread, IsToplevel)
+    ->  true
+    ;   IsToplevel = false
+    ),
+    set_prolog_flag(toplevel_thread, true),
+    call_repl_loop_hook_(begin, BreakLev).
+call_repl_loop_hook(end, BreakLev, IsToplevel) =>
+    set_prolog_flag(toplevel_thread, IsToplevel),
+    call_repl_loop_hook_(end, BreakLev).
+
+call_repl_loop_hook_(BeginEnd, BreakLev) :-
     forall(prolog:repl_loop_hook(BeginEnd, BreakLev), true).
 
 
