@@ -155,7 +155,7 @@ RedrawAreaText(TextObj t, Area a)
 
 
 void
-str_format(String out, const String in, const int width, const FontObj font)
+str_format(PceString out, const PceString in, const int width, const FontObj font)
 { int x = 0;
   int last_is_layout = TRUE;
 
@@ -239,7 +239,7 @@ str_format(String out, const String in, const int width, const FontObj font)
 
 
 void
-str_one_line(String to, String from)
+str_one_line(PceString to, PceString from)
 { int n;
 
   for(n=0; n<from->s_size; n++)
@@ -255,18 +255,20 @@ str_one_line(String to, String from)
 }
 
 
-static void
-draw_caret(int x, int y, int w, int h, int active)
+void
+draw_caret(double x, double y, double w, double h, bool active)
 { if ( active )
-  { int cx = x + w/2;
+  { double cx = x + w/2.0;
+    Colour c = getClassVariableValueClass(ClassTextCursor, NAME_colour);
 
-    r_fillpattern(BLACK_IMAGE, NAME_foreground);
+    r_fillpattern(c, NAME_foreground);
     r_fill_triangle(cx, y, x, y+h, x+w, y+h);
   } else
-  { ipoint pts[4];
-    int cx = x + w/2;
+  { fpoint pts[4];
+    double cx = x + w/2.0;
+    Colour c = getClassVariableValueClass(ClassTextCursor, NAME_inactiveColour);
 
-    int cy = y + h/2;
+    double cy = y + h/2.0;
     int i = 0;
 
     pts[i].x = cx;  pts[i].y = y;   i++;
@@ -274,19 +276,15 @@ draw_caret(int x, int y, int w, int h, int active)
     pts[i].x = cx;  pts[i].y = y+h; i++;
     pts[i].x = x+w; pts[i].y = cy;  i++;
 
-    r_fillpattern(GREY50_IMAGE, NAME_foreground);
+    r_fillpattern(c, NAME_foreground);
     r_fill_polygon(pts, i);
   }
 }
 
 
-#ifndef OL_CURSOR_SIZE
-#define OL_CURSOR_SIZE	9
-#endif
-
 status
 repaintText(TextObj t, int x, int y, int w, int h)
-{ String s = &t->string->data;
+{ PceString s = &t->string->data;
   int b = valInt(t->border);
   int sf = 0, st = 0;
   int flags = 0;
@@ -334,16 +332,25 @@ repaintText(TextObj t, int x, int y, int w, int h)
     { LocalString(buf, s->s_iswide, s->s_size+1);
 
       str_one_line(buf, s);
-      s = buf;
-    }
-    if ( notNil(t->selection) )
-    { str_selected_string(s, t->font, sf, st, style,
-			  x+valInt(t->x_offset), y, w, h,
-			  t->format, NAME_top);
+      if ( notNil(t->selection) )
+      { str_selected_string(buf, t->font, sf, st, style,
+			    x+valInt(t->x_offset), y, w, h,
+			    t->format, NAME_top);
+      } else
+      { str_string(buf, t->font,
+		   x+valInt(t->x_offset), y, w, h,
+		   t->format, NAME_top, flags);
+      }
     } else
-    { str_string(s, t->font,
-		 x+valInt(t->x_offset), y, w, h,
-		 t->format, NAME_top, flags);
+    { if ( notNil(t->selection) )
+      { str_selected_string(s, t->font, sf, st, style,
+			    x+valInt(t->x_offset), y, w, h,
+			    t->format, NAME_top);
+      } else
+      { str_string(s, t->font,
+		   x+valInt(t->x_offset), y, w, h,
+		   t->format, NAME_top, flags);
+      }
     }
   }
 
@@ -351,16 +358,18 @@ repaintText(TextObj t, int x, int y, int w, int h)
     d_clip_done();
 
   if ( t->show_caret != OFF )
-  { int fh = valInt(getAscentFont(t->font));
-    int active = (t->show_caret == ON);
+  { double fh = valNum(getAscentFont(t->font));
+    bool active = (t->show_caret == ON);
     Any colour = getClassVariableValueClass(ClassTextCursor,
 					    active ? NAME_colour
-					           : NAME_inactiveColour);
+						   : NAME_inactiveColour);
     Any old = r_colour(colour);
+    Int h = getClassVariableValueClass(ClassTextCursor, NAME_height);
+    double ols = h ? valNum(h) : 11;
 
-    draw_caret(valInt(t->x_caret) - OL_CURSOR_SIZE/2 + x - b,
-	       valInt(t->y_caret) + y + fh - b - 3,
-	       OL_CURSOR_SIZE, OL_CURSOR_SIZE,
+    draw_caret(valNum(t->x_caret) - ols/2.0 + x - b,
+	       valNum(t->y_caret) + y + fh - b - 3.0,
+	       ols, ols,
 	       active);
 
     r_colour(old);
@@ -386,7 +395,7 @@ static status
 initAreaText(TextObj t)
 { int tw, x, y, w, h;
   Point pos = t->position;
-  String s = &t->string->data;
+  PceString s = &t->string->data;
   int size = s->s_size;
   int b = valInt(t->border);
 
@@ -407,9 +416,10 @@ initAreaText(TextObj t)
     { LocalString(buf, s->s_iswide, s->s_size + 1);
 
       str_one_line(buf, s);
-      s = buf;
+      str_size(buf, t->font, &tw, &h);
+    } else
+    { str_size(s, t->font, &tw, &h);
     }
-    str_size(s, t->font, &tw, &h);
   }
 
   if ( t->wrap == NAME_clip )
@@ -443,7 +453,7 @@ static status
 initPositionText(TextObj t)
 { int tw, x, y, w, h;
   Point pos = t->position;
-  String s = &t->string->data;
+  PceString s = &t->string->data;
   int b = valInt(t->border);
 
   if ( Wrapped(t) )
@@ -458,9 +468,10 @@ initPositionText(TextObj t)
     { LocalString(buf, s->s_iswide, s->s_size + 1);
 
       str_one_line(buf, s);
-      s = buf;
+      str_size(buf, t->font, &tw, &h);
+    } else
+    { str_size(s, t->font, &tw, &h);
     }
-    str_size(s, t->font, &tw, &h);
   }
 
   if ( t->wrap == NAME_clip )
@@ -570,39 +581,25 @@ getCharacterPositionText(TextObj t, Int chr)
 }
 
 
-static status
-get_char_pos_text(TextObj t, Int chr, int *X, int *Y)
-{ int caret = (isDefault(chr) ? valInt(t->caret) : valInt(chr));
-  int w   = abs((int)valInt(t->area->w));
+static void
+get_char_pos_helper(TextObj t, PceString s, int caret, int *cx, int *cy)
+{ int b = valInt(t->border);
   int ch  = valInt(getHeightFont(t->font));
-  int cx, cy = 0, lw, sl;
-  String s = &t->string->data;
-  int b = valInt(t->border);
-
-  if ( Wrapped(t) )
-  { LocalString(buf, s->s_iswide, Wrapped(t) ? s->s_size + MAX_WRAP_LINES : 0);
-
-    str_format(buf, s, valInt(t->margin), t->font);
-    s = buf;
-  } else if ( t->wrap == NAME_clip )
-  { LocalString(buf, s->s_iswide, s->s_size + 1);
-
-    str_one_line(buf, s);
-    s = buf;
-  }
+  int w   = abs((int)valInt(t->area->w));
+  int lw, sl;
 
   if ( (sl = str_next_rindex(s, caret-1, '\n')) < 0 )
-    sl = 0;
-  else
+  { sl = 0;
+  } else
   { sl++;
-    cy += (str_lineno(s, sl)-1) * ch;
+    *cy += (str_lineno(s, sl)-1) * ch;
   }
 
-  lw = str_width(s, sl, caret, t->font);
+  lw = str_advance(s, sl, caret, t->font);
   w -= 2 * b;
 
   if ( t->format == NAME_left )
-  { cx = lw;
+  { *cx = lw;
   } else
   { int el;
     int rw;
@@ -612,9 +609,31 @@ get_char_pos_text(TextObj t, Int chr, int *X, int *Y)
     rw = str_width(s, caret, el, t->font);
 
     if ( t->format == NAME_center )
-      cx = w/2 - (lw+rw)/2 + lw;
+      *cx = w/2 - (lw+rw)/2 + lw;
     else				/* right */
-      cx = w - rw;
+      *cx = w - rw;
+  }
+}
+
+
+static status
+get_char_pos_text(TextObj t, Int chr, int *X, int *Y)
+{ int caret = (isDefault(chr) ? valInt(t->caret) : valInt(chr));
+  int cx = 0;			/* clang doesn't see this is not needed */
+  int cy = 0;
+  PceString s = &t->string->data;
+  int b = valInt(t->border);
+
+  if ( t->wrap == NAME_clip )
+  { LocalString(buf, s->s_iswide, s->s_size + 1);
+
+    str_one_line(buf, s);
+    get_char_pos_helper(t, s, caret, &cx, &cy);
+  } else
+  { LocalString(buf, s->s_iswide, Wrapped(t) ? s->s_size + MAX_WRAP_LINES : s->s_size+1);
+
+    str_format(buf, s, valInt(t->margin), t->font);
+    get_char_pos_helper(t, s, caret, &cx, &cy);
   }
 
   *X = cx + valInt(t->x_offset) + b;
@@ -626,7 +645,7 @@ get_char_pos_text(TextObj t, Int chr, int *X, int *Y)
 
 Int
 get_pointed_text(TextObj t, int x, int y)
-{ String s = &t->string->data;
+{ PceString s = &t->string->data;
   int ch = valInt(getHeightFont(t->font));
   int b = valInt(t->border);
   int cw, w;
@@ -994,7 +1013,7 @@ typedText(TextObj t, EventId id)
 
 
 static int
-start_of_line(String s, int n)
+start_of_line(PceString s, int n)
 { if ( n > 0 && str_fetch(s, n) == '\n' )
     n--;
 
@@ -1005,7 +1024,7 @@ start_of_line(String s, int n)
 
 
 static int
-end_of_line(String s, int n)
+end_of_line(PceString s, int n)
 { if ( (n = str_next_index(s, n, '\n')) < 0 )
     n = s->s_size;
 
@@ -1014,7 +1033,7 @@ end_of_line(String s, int n)
 
 
 static int
-forward_word(String s, int i, int n)
+forward_word(PceString s, int i, int n)
 { while( n-- > 0 && i < s->s_size )
   { while( i < s->s_size && !isalnum(str_fetch(s, i)) ) i++;
     while( i < s->s_size && isalnum(str_fetch(s, i)) ) i++;
@@ -1025,7 +1044,7 @@ forward_word(String s, int i, int n)
 
 
 static int
-backward_word(String s, int i, int n)
+backward_word(PceString s, int i, int n)
 { while( n-- > 0 && i > 0 )
   { i--;
     while( i > 0 && !isalnum(str_fetch(s, i)) ) i--;
@@ -1120,7 +1139,7 @@ getColumnText(TextObj t)
 
 static status
 endOfLineText(TextObj t, Int arg)
-{ String s = &t->string->data;
+{ PceString s = &t->string->data;
   int caret = valInt(t->caret);
   int n;
 
@@ -1136,7 +1155,7 @@ endOfLineText(TextObj t, Int arg)
 
 static status
 beginningOfLineText(TextObj t, Int arg)
-{ String s = &t->string->data;
+{ PceString s = &t->string->data;
   int caret = valInt(t->caret);
   int n;
 
@@ -1274,7 +1293,7 @@ cutOrBackwardDeleteCharText(TextObj t, Int arg)
 
 static status
 killLineText(TextObj t, Int arg)
-{ String s = &t->string->data;
+{ PceString s = &t->string->data;
   int caret = valInt(t->caret);
   int end, n;
 
@@ -1365,7 +1384,7 @@ openLineText(TextObj t, Int arg)
 { int tms = UArg(t);
 
   if ( tms > 0 )
-  { String nl = str_nl(&t->string->data);
+  { PceString nl = str_nl(&t->string->data);
     LocalString(buf, t->string->data.s_iswide, nl->s_size * tms);
     int i;
 
@@ -1388,7 +1407,7 @@ gosmacsTransposeText(TextObj t)
 
   if ( caret >= 2 )
   { wint_t tmp;
-    String s;
+    PceString s;
 
     deselectText(t);
     prepareEditText(t, DEFAULT);
@@ -1409,7 +1428,7 @@ transposeCharsText(TextObj t)
 
   if ( caret >= 1 )
   { wint_t tmp;
-    String s;
+    PceString s;
 
     deselectText(t);
     prepareEditText(t, DEFAULT);
@@ -1823,4 +1842,3 @@ makeClassText(Class class)
 
   succeed;
 }
-
