@@ -740,12 +740,8 @@ End;
 
 NewClass(cursorobj)
   Name		name;			/* Name of the cursor */
-  Int		font_id;		/* Identifier in X-cursor-font */
   Image		image;			/* User-defined image */
-  Image		mask;			/* User-defined mask */
   Point		hot_spot;		/* User-defined hot_spot */
-  Colour	foreground;		/* User-defined foreground */
-  Colour	background;		/* User-defined background */
   WsRef		ws_ref;			/* Window system reference */
 End;
 
@@ -771,11 +767,11 @@ End;
 NewClass(fontobj)
   Name		family;			/* Font family */
   Name		style;			/* Style */
-  Int		points;			/* Point size */
-  Int		ex;			/* Width of an x in this font */
-  Name          x_name;			/* X-Name of the font */
+  Any		weight;			/* Weight */
+  Num		points;			/* Point size */
+  Num		ex;			/* Width of an x in this font */
+  Num		avg_char_width;		/* Average char width */
   BoolObj	fixed_width;		/* Is this a fixed width font? */
-  BoolObj	iswide;			/* Font is 16-bit font */
   Name		postscript_font;	/* Name of PostScript font */
   Int		postscript_size;	/* Size in PostScript */
   WsRef		ws_ref;			/* Window system reference */
@@ -784,19 +780,16 @@ End;
 
 NewClass(displayobj)
   ABSTRACT_VISUAL
-  Size		size;			/* size of the display in pixels */
+  Name		name;			/* Human name of the display */
+  BoolObj	primary;		/* Display is the primary display */
+  Area		area;			/* Area occupied by this display */
+  Area		work_area;		/* Area for applications */
+  BoolObj	removed;		/* Display is being removed */
   Size		dpi;			/* Resolution (dots per inch) */
-  Name		address;		/* Display address specification */
-  HashTable	font_table;		/* Table holding font-mappings */
   Chain		frames;			/* Created frames on this display */
-  Chain		monitors;		/* Physical monitors attached */
   Chain		inspect_handlers;	/* Event-handlers for inspector */
   Colour	foreground;		/* Window default foreground */
   Colour	background;		/* Window default background */
-  Name		wm_class;		/* WM_CLASS hint */
-  BoolObj	quick_and_dirty;	/* Use quick_and_dirty drawing */
-  Image		cache;			/* Graphics cache */
-  Name		window_manager;		/* name of the window manager */
   DisplayManager display_manager;	/* Global display manager */
   Int		busy_locks;		/* Lock count of ->busy_cursor */
   WsRef		ws_ref;			/* Window System Reference */
@@ -806,7 +799,6 @@ End;
 NewClass(display_manager)
   ABSTRACT_VISUAL
   Chain		members;		/* Available displays */
-  Chain		current;		/* Stack for current display */
   BoolObj	test_queue;		/* Test queue during redraw */
 End;
 
@@ -926,17 +918,10 @@ NewClass(timer)
   Real		interval;		/* interval as a real value */
   Code		message;		/* message when timer fires */
   Name		status;			/* {idle,interval,once} */
-  BoolObj		service;		/* Operating in service mode? */
+  BoolObj	service;		/* Operating in service mode? */
   WsRef		ws_ref;			/* Window System Reference */
 End;
 
-NewClass(monitor)
-  Any		name;			/* Name of the physical monitor */
-  Area		area;			/* Area on the display */
-  Area		work_area;		/* Application area */
-  BoolObj	primary;		/* Monitor is primary */
-End;
-
 		 /*******************************
 		 *	      EVENTS		*
 		 *******************************/
@@ -1109,6 +1094,8 @@ GLOBAL  int XrefsResolved;		/* succesful getXrefObject()'s */
 GLOBAL	HashTable ColourTable;		/* ColourName --> Colour */
 GLOBAL  HashTable CursorTable;		/* CursorName --> Cursor */
 GLOBAL	HashTable FontTable;		/* FontName --> Font */
+GLOBAL  HashTable FontAliasTable;	/* Alias -> font */
+GLOBAL  HashTable FontFamilyTable;	/* Family -> Pango family */
 GLOBAL	HashTable ImageTable;		/* ImageName --> Image */
 GLOBAL  HashTable WindowTable;		/* X-Window --> PceWindow|FrameObj */
 GLOBAL  Class ClassImage;		/* @image_class */
