@@ -205,6 +205,60 @@ static Any grabbing_window = NIL;
 static Any mouse_tracking_window = NIL; /* Window or Frame */
 static Uint8 mouse_tracking_button;
 static SDL_WindowID mouse_tracking_wid = 0;
+static SDL_DisplayID last_display_id = 0;
+static Uint32 keyboard_timer = 0;
+static SDL_Event keydown_event = {0};
+static Uint64 keyinput_time = 0;
+
+static Uint32
+tm_keyboard_timeout(void *udata, SDL_TimerID id, Uint32 interval)
+{ keyboard_timer = 0;
+
+  if ( keyinput_time < keydown_event.key.timestamp )
+  { SDL_Event ev;
+
+    SDL_zero(ev);
+    ev.type = MY_EVENT_KEYDOWN_TIMEOUT;
+    SDL_PushEvent(&ev);
+    DEBUG(NAME_keyboard,
+	  Cprintf("%"PRIu64" < %"PRIu64": Pushed delayed keyboard event\n",
+		  keyinput_time, keydown_event.key.timestamp));
+  }
+
+  return 0;
+}
+
+/** True if the character should be passed as a printable
+ *  character rather than a keydown event with a modifier.
+ */
+
+static StringObj macOSOptionCharacters = NULL;
+
+static bool
+isOptionPrintCharacter(int code)
+{ if ( code >= 32 && code <= 126 )
+    return true;
+
+  if ( !macOSOptionCharacters &&
+       !isNil(macOSOptionCharacters) )
+  { macOSOptionCharacters = getClassVariableValueClass(
+      ClassEvent,
+      NAME_macosOptionCharacters);
+    if ( macOSOptionCharacters )
+    { DEBUG(NAME_keyboard,
+	    Cprintf("macOSOptionCharacters = %s\n",
+		    pp(macOSOptionCharacters)));
+    } else
+    { macOSOptionCharacters = (StringObj)NIL;
+    }
+  }
+
+  if ( notNil(macOSOptionCharacters) )
+    return str_index(&macOSOptionCharacters->data, code) != -1;
+
+  return false;
+}
+
 
 void
 ev_event_grab_window(Any window)
@@ -219,9 +273,15 @@ ws_event_destroyed_target(Any window)
     grabbing_window = NIL;
 }
 
+DisplayObj
+ws_last_display_from_event(void)
+{ return dsp_id_to_display(last_display_id);
+}
+
 EventObj
 CtoEvent(SDL_Event *event)
-{ unsigned int time;
+{ ASSERT_SDL_MAIN();
+  unsigned int time;
   SDL_MouseButtonFlags mouse_flags = 0;
   float fx, fy;
   Any name = NULL;
@@ -233,6 +293,8 @@ CtoEvent(SDL_Event *event)
 
   if ( sdl_call_event(event) )	/* support in_pce_thread/1 */
     fail;
+  if ( sdl_display_event(event) ) /* display events (add/delete/change) */
+    fail;
   if ( sdl_frame_event(event) )	/* window events (close/open/size/...) */
     fail;
   if ( sdl_timer_event(event) )	/* Timer event */
@@ -277,17 +339,21 @@ CtoEvent(SDL_Event *event)
 	name = NAME_locMove;
       break;
     case SDL_EVENT_MOUSE_WHEEL:
+    { static int last_time = 0;
       wid  = event->wheel.windowID;
       time = event->wheel.timestamp/1000000;
       name = NAME_wheel;
       ctx_name = NAME_rotation;
-      float dx = event->wheel.x;
+      //float dx = event->wheel.x;
       float dy = event->wheel.y;
 
-      if ( event->wheel.direction == SDL_MOUSEWHEEL_FLIPPED )
-      { dx = -dx;
-        dy = -dy;
-      }
+      DEBUG(NAME_wheel,
+	    Cprintf("Mouse wheel event.  dy=%.6f, dt=%dms%s\n",
+		    dy, time-last_time,
+		    event->wheel.direction == SDL_MOUSEWHEEL_FLIPPED
+		    ? " (flipped)" : ""));
+      last_time = time;
+
       if ( dy > 0.0 )
 	ctx = toInt(120);
       else if ( dy < 0.0 )
@@ -295,31 +361,69 @@ CtoEvent(SDL_Event *event)
       else
 	fail;
       break;
+    }
       /* https://wiki.libsdl.org/SDL3/SDL_KeyboardEvent */
     case SDL_EVENT_KEY_UP:
       lastmod = event->key.mod;
       fail;		      /* only update modifiers */
       /* https://wiki.libsdl.org/SDL3/SDL_TextInputEvent */
-    case SDL_EVENT_TEXT_INPUT: /* Needed for input composition.  TBD */
-    { SDL_Keymod mod = SDL_GetModState();
-      if ( mod & (ControlMask|MetaMask|SDL_KMOD_GUI) )
-	fail;
-      int codepoint;
+    case SDL_EVENT_TEXT_INPUT:
+    { int codepoint;
       char const *u = event->text.text;
       u = utf8_get_char(u, &codepoint);
       if ( *u )
         Cprintf("SDL_EVENT_TEXT_INPUT: multi-char input not yet supported\n");
+
+      DEBUG(NAME_keyboard,
+	    Cprintf("SDL_EVENT_TEXT_INPUT: %s at %" PRIu64 "\n",
+		    event->text.text, event->text.timestamp));
+
+      if ( keyboard_timer )
+      { if ( isOptionPrintCharacter(codepoint) )
+	{ SDL_RemoveTimer(keyboard_timer);
+	  keyboard_timer = 0;
+	  keyinput_time = event->text.timestamp;
+	} else
+	{ fail;			/* process as key-down */
+	}
+      }
+
       wid     = event->text.windowID;
       time    = event->text.timestamp/1000000;
       name    = toInt(codepoint);
       break;
     }
     case SDL_EVENT_KEY_DOWN:
+    { SDL_Keymod isdown = (SDL_KMOD_LCTRL|SDL_KMOD_RCTRL|SDL_KMOD_GUI);
+#ifndef __APPLE__
+      isdown |= SDL_KMOD_LALT|SDL_KMOD_RALT;
+#endif
+
+      name = keycode_to_name(event);
+      if ( !name ) fail;
+
+      if ( event->key.mod & isdown )
+	goto immediate_down_event;
+
+      keydown_event  = *event;
+      keyinput_time  = 0;
+      keyboard_timer = SDL_AddTimer(10, tm_keyboard_timeout, NULL);
+      DEBUG(NAME_keyboard,
+	    Cprintf("Delaying keyboard down event at %" PRIu64 "\n"));
+      fail;
+    }
+    case MY_EVENT_KEYDOWN_TIMEOUT:
+      event   = &keydown_event;
+    immediate_down_event:
       wid     = event->key.windowID;
       time    = event->key.timestamp/1000000;
       lastmod = event->key.mod;
       name    = keycode_to_name(event);
-      if ( !name ) fail;
+      DEBUG(NAME_keyboard,
+	    bool delayed = (&keydown_event == event);
+	    Cprintf("SDL_EVENT_KEY_DOWN: %s at %"PRIu64"%s\n",
+		    pp(name), event->text.timestamp,
+		    delayed ? " (delayed)" : ""));
       break;
     default:
       fail;			/* for now */
@@ -342,6 +446,11 @@ CtoEvent(SDL_Event *event)
   float scale = 1.0;
 #endif
 
+  if ( name != NAME_locMove )
+  { WsFrame wfr = sdl_frame(frame, false);
+    last_display_id = SDL_GetDisplayForWindow(wfr->ws_window);
+  }
+
   float x = fx*scale;
   float y = fy*scale;
   if ( notNil(mouse_tracking_window) )
@@ -476,7 +585,8 @@ set_watch(waitable_t fd)
 
 static bool
 dispatch_ready_event(void)
-{ SDL_Event ev;
+{ ASSERT_SDL_MAIN();
+  SDL_Event ev;
 
   if ( SDL_PollEvent(&ev) )
   { EventObj event = CtoEvent(&ev);
