@@ -80,7 +80,8 @@ ws_created_frame(FrameObj fr)
 
 static void
 uncreate_window_frame(PceWindow sw)
-{ WsWindow wsw = sw->ws_ref;
+{ ASSERT_SDL_MAIN();
+  WsWindow wsw = sw->ws_ref;
 
   if ( wsw->texture )
   { SDL_DestroyTexture(wsw->texture);
@@ -117,7 +118,8 @@ ws_uncreate_frame(FrameObj fr)
 { WsFrame f = sdl_frame(fr, false);
 
   if ( f && f->ws_window )
-  { deleteChain(ChangedFrames, fr);
+  { ASSERT_SDL_MAIN();
+    deleteChain(ChangedFrames, fr);
     SDL_DestroyRenderer(f->ws_renderer);
     SDL_DestroyWindow(f->ws_window);
     unalloc(sizeof(*f), f);
@@ -129,13 +131,15 @@ ws_uncreate_frame(FrameObj fr)
 }
 
 static SDL_Window *
-sdl_parent_window(FrameObj fr)
+sdl_parent_window(FrameObj fr, FrameObj *frp)
 { Any pfr = fr->transient_for;
 
   if ( isNil(pfr) )
     pfr = getAttributeObject(fr, NAME_parent);
   if ( pfr && instanceOfObject(pfr, ClassFrame) )
-  { WsFrame pf = sdl_frame(pfr, false);
+  { if ( frp )
+      *frp = pfr;
+    WsFrame pf = sdl_frame(pfr, false);
     if ( pf )
       return pf->ws_window;
   }
@@ -151,25 +155,39 @@ sdl_parent_window(FrameObj fr)
  */
 status
 ws_create_frame(FrameObj fr)
-{ SDL_Window *win = NULL;
-  SDL_Window *parent = sdl_parent_window(fr);
+{ ASSERT_SDL_MAIN();
+  SDL_Window *win = NULL;
+  FrameObj pfr = NIL;
+  SDL_Window *parent = sdl_parent_window(fr, &pfr);
   int x = valInt(fr->area->x);
   int y = valInt(fr->area->y);
   int w = valInt(fr->area->w);
   int h = valInt(fr->area->h);
 
   if ( fr->kind == NAME_popup && parent )
-  { SDL_WindowFlags flags = 0;
+  { SDL_PropertiesID props = SDL_CreateProperties();
+    SDL_SetPointerProperty(props, SDL_PROP_WINDOW_CREATE_PARENT_POINTER,
+			   parent);
+    SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_MENU_BOOLEAN, true);
+#if defined(__APPLE__) && defined(SDL_PROP_WINDOW_CREATE_CONSTRAIN_POPUP_BOOLEAN)
+    /* SDL on MacOS does not handle popup placement correctly on secondary displays */
+    SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_CONSTRAIN_POPUP_BOOLEAN, false);
+#endif
+    SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_X_NUMBER, x);
+    SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, y);
+    SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_WIDTH_NUMBER, w);
+    SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_HEIGHT_NUMBER, h);
+#if O_HDP
+    SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_HIGH_PIXEL_DENSITY_BOOLEAN,
+			   true);
+#endif
 
-#if O_HDP
-    flags = SDL_WINDOW_HIGH_PIXEL_DENSITY;
-    //float scale = SDL_GetWindowPixelDensity(parent);
-    //x = x/scale; y = y/scale; w = w/scale; h = h/scale;
+    win = SDL_CreateWindowWithProperties(props);
+    SDL_DestroyProperties(props);
+#ifdef __APPLE__
+    if ( win )
+      SDL_SetWindowPosition(win, x, y);
 #endif
-    DEBUG(NAME_popup,
-	  Cprintf("Creating popup frame %s %dx%d\n", pp(fr), w, h));
-    flags |= SDL_WINDOW_POPUP_MENU;
-    win = SDL_CreatePopupWindow(parent, x, y, w, h, flags);
   } else
   {
 #if O_HDPX
@@ -197,14 +215,27 @@ ws_create_frame(FrameObj fr)
     { SDL_SetPointerProperty(props, SDL_PROP_WINDOW_CREATE_PARENT_POINTER,
 			     parent);
     }
-    if ( fr->placed == ON )
+    if ( isOn(fr->placed) )
     {
 #ifdef __WINDOWS__
       x += GetSystemMetrics(SM_CXBORDER);
       y += GetSystemMetrics(SM_CYBORDER) + GetSystemMetrics(SM_CYCAPTION);
 #endif
+      DisplayObj dsp = fr->display;
+      x += valInt(dsp->work_area->x);
+      y += valInt(dsp->work_area->y);
+      DEBUG(NAME_frame,
+	    Cprintf("%s: passing position %d,%d\n", pp(fr), x, y));
       SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_X_NUMBER, x);
       SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, y);
+    } else
+    { DisplayObj dsp = fr->display;
+      if ( !isOn(dsp->primary) )
+      { x = valInt(dsp->work_area->x) + (valInt(dsp->work_area->w)-w)/2;
+	y = valInt(dsp->work_area->y) + (valInt(dsp->work_area->h)-h)/2;
+	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_X_NUMBER, x);
+	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, y);
+      }
     }
     win = SDL_CreateWindowWithProperties(props);
     SDL_DestroyProperties(props);
@@ -365,7 +396,8 @@ typedef struct
 
 static void*
 ws_draw_resize_area_frame(Any ctx, TileObj t, Int x, Int y, Int w, Int h)
-{ FrameObj fr = ctx;
+{ ASSERT_SDL_MAIN();
+  FrameObj fr = ctx;
   WsFrame wfr = fr->ws_ref;
   float x1, y1, x2, y2;
 
@@ -396,7 +428,8 @@ ws_draw_resize_area_frame(Any ctx, TileObj t, Int x, Int y, Int w, Int h)
 
 static void
 ws_draw_resize_frame(FrameObj fr)
-{ TileObj tile = getTileFrame(fr);
+{ ASSERT_SDL_MAIN();
+  TileObj tile = getTileFrame(fr);
 
   if ( tile )
   { WsFrame wfr = fr->ws_ref;
@@ -415,7 +448,8 @@ ws_draw_window(FrameObj fr, PceWindow sw, foffset *off)
   WsWindow wsw = sw->ws_ref;
 
   if ( wsw )
-  { Area a = sw->area;
+  { ASSERT_SDL_MAIN();
+    Area a = sw->area;
     SDL_FRect dstrect = Area2FRect(a);
     float scale = SDL_GetWindowPixelDensity(wfr->ws_window);
 
@@ -485,6 +519,7 @@ ws_draw_frame(FrameObj fr)
     false;
 
   WsFrame wfr = fr->ws_ref;
+  ASSERT_SDL_MAIN();
 
   DEBUG(NAME_sdl,
 	Cprintf("BEGIN ws_draw_frame(%s)\n", pp(fr)));
@@ -551,6 +586,15 @@ sdl_frame_event(SDL_Event *ev)
       }
       case SDL_EVENT_WINDOW_SHOWN:
 	DEBUG(NAME_sdl, Cprintf("Mapped %s\n", pp(fr)));
+	WsFrame wfr = fr->ws_ref;
+	if ( wfr )
+	{ SDL_DisplayID did = SDL_GetDisplayForWindow(wfr->ws_window);
+	  DisplayObj dsp = dsp_id_to_display(did);
+	  if ( dsp && dsp != fr->display )
+	  { DEBUG(NAME_display, Cprintf("Opened %s on %s\n", pp(fr), pp(dsp)));
+	    assign(fr, display, dsp);
+	  }
+	}
 	return frame_displayed(fr, ON);
       case SDL_EVENT_WINDOW_HIDDEN:
 	//return frame_displayed(fr, OFF);
@@ -594,6 +638,12 @@ sdl_frame_event(SDL_Event *ev)
       case SDL_EVENT_WINDOW_FOCUS_LOST:
 	DEBUG(NAME_keyboard, Cprintf("Input focus lost for %s\n", pp(fr)));
 	return send(fr, NAME_inputFocus, OFF, EAV);
+      case SDL_EVENT_WINDOW_DISPLAY_CHANGED:
+      { DisplayObj new_display = dsp_id_to_display(ev->window.data1);
+	DEBUG(NAME_display, Cprintf("%s moved to %s\n",
+				    pp(fr), pp(new_display)));
+	return send(fr, NAME_display, new_display, EAV);
+      }
     }
   }
 
@@ -631,7 +681,9 @@ void
 ws_raise_frame(FrameObj fr)
 { WsFrame wfr = fr->ws_ref;
   if ( wfr && wfr->ws_window )
+  { ASSERT_SDL_MAIN();
     SDL_RaiseWindow(wfr->ws_window);
+  }
 }
 
 /**
@@ -679,7 +731,9 @@ void
 ws_frame_cursor(FrameObj fr, CursorObj cursor)
 { SDL_Cursor *c = pceCursor2SDL_Cursor(cursor);
   if ( c )
+  { ASSERT_SDL_MAIN();
     SDL_SetCursor(c);
+  }
 }
 
 /**
@@ -707,7 +761,8 @@ ws_enable_text_input(Graphical gr, BoolObj enable)
   { WsFrame wfr = fr->ws_ref;
 
     if ( wfr && wfr->ws_window )
-    { DEBUG(NAME_keyboard,
+    { ASSERT_SDL_MAIN();
+      DEBUG(NAME_keyboard,
 	    Cprintf("ws_enable_text_input() %s -> %s: %s\n",
 		    pp(gr), pp(fr), pp(enable)));
       if ( isOn(enable) )
@@ -750,14 +805,14 @@ ws_frame_bb(FrameObj fr, int *x, int *y, int *w, int *h)
  *
  * @param fr Pointer to the FrameObj.
  * @param spec Name object containing the geometry specification.
- * @param mon Monitor object representing the target monitor.
+ * @param dsp Display object representing the target monitor.
  */
 #define MIN_VISIBLE 32			/* pixels that must be visible */
 #define WIN_NOMOVE 0x1
 #define WIN_NOSIZE 0x2
 
 void
-ws_x_geometry_frame(FrameObj fr, Name spec, Monitor mon)
+ws_x_geometry_frame(FrameObj fr, Name spec, DisplayObj dsp)
 { char *e, *s = strName(spec);
   int x, y, w, h, w0, h0;
   int eh;
@@ -768,15 +823,15 @@ ws_x_geometry_frame(FrameObj fr, Name spec, Monitor mon)
   Int X,Y,W,H;
   int offX=0;			/* window manager frame offset */
 
-  if ( isDefault(mon) && (e=strchr(s, '@')) )
+  if ( isDefault(dsp) && (e=strchr(s, '@')) )
   { int n = atoi(e+1);
 
-    if ( !(mon = getNth0Chain(fr->display->monitors, toInt(n))) )
-      mon = (Monitor)DEFAULT;
+    if ( !(dsp=getNth0Chain(fr->display->display_manager->members, toInt(n))) )
+      dsp = DEFAULT;
   }
 
-  if ( instanceOfObject(mon, ClassMonitor) )
-  { Area a = (notNil(mon->work_area) ? mon->work_area : mon->area);
+  if ( instanceOfObject(dsp, ClassDisplay) )
+  { Area a = (notNil(dsp->work_area) ? dsp->work_area : dsp->area);
 
     dw = valInt(a->w);
     dh = valInt(a->h);
@@ -855,7 +910,7 @@ ws_x_geometry_frame(FrameObj fr, Name spec, Monitor mon)
     H = toInt(h);
   }
 
-  send(fr, NAME_set, X, Y, W, H, mon, EAV);
+  send(fr, NAME_set, X, Y, W, H, dsp, EAV);
 }
 
 /**
@@ -866,11 +921,49 @@ ws_x_geometry_frame(FrameObj fr, Name spec, Monitor mon)
  * @param y Y-coordinate position.
  * @param w Width of the frame.
  * @param h Height of the frame.
- * @param mon Monitor object representing the target monitor.
+ * @param dsp Display object representing the target monitor.
  */
-void
-ws_geometry_frame(FrameObj fr, Int x, Int y, Int w, Int h, Monitor mon)
-{
+status
+ws_geometry_frame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj dsp)
+{ WsFrame wsf = fr->ws_ref;
+
+  if ( wsf )
+  {  if ( notDefault(w) || notDefault(h) )
+    { int iw = isDefault(w) ? valInt(fr->area->w) : valInt(w);
+      int ih = isDefault(h) ? valInt(fr->area->h) : valInt(h);
+
+#if O_HDPX
+      float scale = ws_pixel_density_display(fr);
+      iw = iw/scale; ih = ih/scale;
+#endif
+      DEBUG(NAME_set,
+	    Cprintf("SDL_SetWindowSize(%s, %d, %d)\n",
+		    pp(fr), iw, ih));
+      ASSERT_SDL_MAIN();
+      if ( !SDL_SetWindowSize(wsf->ws_window, iw, ih) )
+	Cprintf("Could not set size of %s: %s\n",
+		pp(fr), SDL_GetError());
+    }
+
+    if ( notDefault(x) || notDefault(y) )
+    { int ix = isDefault(x) ? valInt(fr->area->x) : valInt(x);
+      int iy = isDefault(y) ? valInt(fr->area->y) : valInt(y);
+
+#if O_HDPX
+      float scale = ws_pixel_density_display(fr);
+      ix = ix/scale; iy = iy/scale;
+#endif
+      DEBUG(NAME_set,
+	    Cprintf("SDL_SetWindowPosition(%s, %d, %d)\n",
+		    pp(fr), ix, iy));
+      ASSERT_SDL_MAIN();
+      if ( !SDL_SetWindowPosition(wsf->ws_window, ix, iy) )
+	Cprintf("Could not set size of %s: %s\n",
+		pp(fr), SDL_GetError());
+    }
+  }
+
+  succeed;
 }
 
 /**
@@ -978,6 +1071,7 @@ ws_status_frame(FrameObj fr, Name status)
   } else if ( status == NAME_window || status == NAME_fullScreen )
   { if ( ws_created_frame(fr) )
     { WsFrame wfr = fr->ws_ref;
+      ASSERT_SDL_MAIN();
       SDL_SetWindowFullscreen(wfr->ws_window, status == NAME_fullScreen);
     } else
     { assign(fr, status, status);
@@ -1006,7 +1100,8 @@ void
 ws_set_label_frame(FrameObj fr)
 { WsFrame wfr = fr->ws_ref;
   if ( wfr && wfr->ws_window )
-  { SDL_SetWindowTitle(wfr->ws_window, nameToUTF8(fr->label));
+  { ASSERT_SDL_MAIN();
+    SDL_SetWindowTitle(wfr->ws_window, nameToUTF8(fr->label));
   }
 }
 
