@@ -605,7 +605,10 @@ fontTerminalImage(TerminalImage ti, FontObj font, FontObj bold)
     bold = NIL;
   }
   assign(ti, bold_font, bold);
-  return refreshTerminalImage(ti);
+  requestComputeGraphical(ti, DEFAULT);
+  changedEntireImageGraphical(ti);
+
+  succeed;
 }
 
 static status
@@ -820,9 +823,9 @@ static classvardecl rc_terminal_image[] =
      "How many lines are saved for scroll back"),
   RC(NAME_syntax, "[syntax_table]", "default",
      "Syntax definition"),
-  RC(NAME_font, "font", "font(screen,roman,13)",
+  RC(NAME_font, "font", "tt",
      "Default font"),
-  RC(NAME_boldFont, "font*", "font(screen,bold,13)",
+  RC(NAME_boldFont, "font*", "boldtt",
      "Bold font"),
   RC(NAME_ansiColours, "vector*",
      "vector("
@@ -1725,7 +1728,7 @@ rlc_request_redraw(RlcData b)
   } else
   { int i = b->window_start;
     int y = 0;
-    int ymin, ymax;
+    int ymin=0, ymax;
     bool first = true;
 
     for(; y < b->window_size; y++, i = NextLine(b, i))
@@ -1792,12 +1795,10 @@ rlc_resize_pixel_units(RlcData b, int w, int h)
 
 static void
 rlc_init_text_dimensions(RlcData b, FontObj font)
-{ b->cw = c_width('m', font);
+{ b->cw = valNum(getAvgCharWidthFont(font));
   b->cb = s_ascent(font);
   b->ch = s_height(font);
   b->fixedfont = font->fixed_width == ON;
-
-  //rlc_resize_pixel_units(b, rect.right - rect.left, rect.bottom - rect.top);
 }
 
 
@@ -2864,7 +2865,8 @@ rlc_clear_dec_mode(RlcData b, int mode)
 static void
 osc_command(RlcData b, int param, const uchar_t *link)
 { switch(param)
-  { case 0:
+  { case 0:			/* should set title and icon name */
+    case 2:			/* only sets title */
     { AnswerMark mark;
       StringObj s = TCHAR2String(link);
       markAnswerStack(mark);
@@ -3592,6 +3594,7 @@ getPrologStreamTerminalImage(Any obj,
 
 #elif __WINDOWS__
 #include <msw/mswin.h>
+#define PIPE_SIZE 8192
 
 static bool
 rlc_create_pipes(RlcData b)
@@ -3599,9 +3602,9 @@ rlc_create_pipes(RlcData b)
     return true;
 
   if ( !CreatePipeEx(&b->ptycon.hIn, &b->ptycon.hTaskOut, NULL,
-		     0, FILE_FLAG_OVERLAPPED, 0) ||
+		     PIPE_SIZE, 0, FILE_FLAG_OVERLAPPED) ||
        !CreatePipeEx(&b->ptycon.hTaskIn, &b->ptycon.hOut,  NULL,
-		     FILE_FLAG_OVERLAPPED, 0, 0) )
+		     PIPE_SIZE, FILE_FLAG_OVERLAPPED, 0) )
   { Cprintf("Failed to create ptyCon pipes\n");
     return false;
   }
