@@ -1,9 +1,9 @@
 /*  Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        jan@swi-prolog.org
     WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (c)  1995-2023, University of Amsterdam
+    Copyright (c)  1995-2025, University of Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
@@ -43,7 +43,6 @@
 static status
 initialiseDisplayManager(DisplayManager dm)
 { assign(dm, members, newObject(ClassChain, EAV));
-  assign(dm, current, newObject(ClassChain, EAV));
 
   protectObject(dm);
 
@@ -53,22 +52,18 @@ initialiseDisplayManager(DisplayManager dm)
 
 status
 appendDisplayManager(DisplayManager dm, DisplayObj d)
-{ appendChain(dm->members, d);
-  if ( emptyChain(dm->current) )
-    prependChain(dm->current, d);
-
-  succeed;
+{ return appendChain(dm->members, d);
 }
 
 
 DisplayObj
-getMemberDisplayManager(DisplayManager dm, Name address)
+getMemberDisplayManager(DisplayManager dm, Name name)
 { Cell cell;
 
   for_cell(cell, dm->members)
   { DisplayObj d = cell->value;
 
-    if ( d->address == address )
+    if ( d->name == name )
       answer(d);
   }
 
@@ -76,45 +71,48 @@ getMemberDisplayManager(DisplayManager dm, Name address)
 }
 
 
-static status
-currentDisplayManager(DisplayManager dm, DisplayObj d)
-{ return prependChain(dm->current, d);
+status
+deleteDisplayManager(DisplayManager dm, DisplayObj d)
+{ return deleteChain(dm->members, d);
+}
+
+
+static DisplayObj
+getPrimaryDisplayManager(DisplayManager dm)
+{ Cell cell;
+
+  for_cell(cell, dm->members)
+  { DisplayObj dsp = cell->value;
+
+    if ( isOn(dsp->primary) )
+      answer(dsp);
+  }
+
+  answer(getHeadChain(dm->members));
 }
 
 
 static DisplayObj
 getCurrentDisplayManager(DisplayManager dm)
-{ if ( emptyChain(dm->current) )
-  { realiseClass(ClassDisplay);
+{ DisplayObj dsp = ws_last_display_from_event();
 
-    if ( emptyChain(dm->current) )
-    { errorPce(dm, NAME_noCurrentDisplay);
-      fail;
-    }
-  }
-
-  answer(dm->current->head->value);
-}
-
-
-static status
-popCurrentDisplayManager(DisplayManager dm)
-{ if ( getSizeChain(dm->current) == ONE )
-    return errorPce(dm, NAME_stackEmpty, NAME_current);
-
-  return deleteHeadChain(dm->current);
+  if ( dsp )
+    answer(dsp);
+  answer(getPrimaryDisplayManager(dm));
 }
 
 
 DisplayObj
 CurrentDisplay(Any obj)
-{ DisplayObj d;
+{ DisplayObj dsp;
 
+  if ( instanceOfObject(obj, ClassDisplay) )
+    return obj;
   if ( instanceOfObject(obj, ClassGraphical) &&
-       (d = getDisplayGraphical((Graphical) obj)) )
-    return d;
+       (dsp = getDisplayGraphical((Graphical) obj)) )
+    return dsp;
 
-  return getCurrentDisplayManager(TheDisplayManager());
+  answer(getCurrentDisplayManager(TheDisplayManager()));
 }
 
 
@@ -230,7 +228,7 @@ getContainsDisplayManager(DisplayManager dm)
 		 *******************************/
 
 DisplayManager
-TheDisplayManager()
+TheDisplayManager(void)
 { static DisplayManager dm = NULL;
 
   if ( !dm )
@@ -239,6 +237,24 @@ TheDisplayManager()
   return dm;
 }
 
+static status
+hasVisibleFramesDisplayManager(DisplayManager dm)
+{ if ( notNil(dm->members) )
+  { Cell cell;
+
+    for_cell(cell, dm->members)
+    { DisplayObj dsp = cell->value;
+      if ( !onFlag(dsp, F_FREED|F_FREEING) )
+      { if ( hasVisibleFramesDisplay(dsp) )
+	  succeed;
+      }
+    }
+  }
+
+  fail;
+}
+
+
 
 		 /*******************************
 		 *	 CLASS DECLARATION	*
@@ -249,8 +265,6 @@ TheDisplayManager()
 static vardecl var_displayManager[] =
 { IV(NAME_members, "chain", IV_GET,
      NAME_display, "Available displays"),
-  IV(NAME_current, "chain", IV_NONE,
-     NAME_current, "Stack with current displays"),
   IV(NAME_testQueue, "bool", IV_BOTH,
      NAME_event, "Test queue in event-loop")
 };
@@ -260,14 +274,12 @@ static vardecl var_displayManager[] =
 static senddecl send_displayManager[] =
 { SM(NAME_initialise, 0, NULL, initialiseDisplayManager,
      DEFAULT, "Create the display manager"),
-  SM(NAME_current, 1, "display", currentDisplayManager,
-     NAME_current, "Make display the current display"),
-  SM(NAME_popCurrent, 0, NULL, popCurrentDisplayManager,
-     NAME_current, "Pop the current stack"),
   SM(NAME_append, 1, "display", appendDisplayManager,
      NAME_display, "Attach a new display to the manager"),
   SM(NAME_redraw, 0, NULL, redrawDisplayManager,
-     NAME_event, "Flush all pending changes to the screen")
+     NAME_event, "Flush all pending changes to the screen"),
+  SM(NAME_hasVisibleFrames, 0, NULL, hasVisibleFramesDisplayManager,
+     NAME_organisation, "True if there is at least one visible frame")
 };
 
 /* Get Methods */
@@ -275,11 +287,14 @@ static senddecl send_displayManager[] =
 static getdecl get_displayManager[] =
 { GM(NAME_contains, 0, "chain", NULL, getContainsDisplayManager,
      DEFAULT, "Contained displays"),
+  GM(NAME_primary, 0, "display", NULL, getPrimaryDisplayManager,
+     NAME_display, "Get the primary display"),
   GM(NAME_current, 0, "display", NULL, getCurrentDisplayManager,
      NAME_current, "Get the current display"),
   GM(NAME_member, 1, "display", "name", getMemberDisplayManager,
-     NAME_display, "Find display for specified address"),
-  GM(NAME_windowOfLastEvent, 0, "window", NULL, getWindowOfLastEventDisplayManager,
+     NAME_display, "Find display from name"),
+  GM(NAME_windowOfLastEvent, 0, "window", NULL,
+     getWindowOfLastEventDisplayManager,
      NAME_event, "Find window that received last event")
 };
 
