@@ -1,9 +1,10 @@
 /*  Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        jan@swi-prolog.org
     WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (c)  1985-2002, University of Amsterdam
+    Copyright (c)  1985-2025, University of Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,14 +37,14 @@
 #include <h/graphics.h>
 
 TileObj		getTileFrame(FrameObj);
-forwards int	get_position_from_center_frame(FrameObj, Monitor, Point, int *, int *);
-static void	ensure_on_display(FrameObj, Monitor, int *, int *);
+forwards int	get_position_from_center_frame(FrameObj, DisplayObj, Point, int *, int *);
+static void	ensure_on_display(FrameObj, DisplayObj, int *, int *);
 static status	closedFrame(FrameObj, BoolObj);
-static status	openFrame(FrameObj fr, Point pos, BoolObj grab, BoolObj normalise);
+static status	openFrame(FrameObj fr, Point pos, DisplayObj dsp,
+			  BoolObj grab);
 static status	doneMessageFrame(FrameObj fr, Code msg);
-static status	geometryFrame(FrameObj fr, Name spec, Monitor mon);
-static status	setFrame(FrameObj fr, Int x, Int y, Int w, Int h, Monitor mon);
-static status	flushFrame(FrameObj fr);
+static status	geometryFrame(FrameObj fr, Name spec, DisplayObj mon);
+static status	setFrame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj mon);
 static status	kindFrame(FrameObj fr, Name kind);
 static status	informTransientsFramev(FrameObj fr, Name selector,
 				       int argc, Any *argv);
@@ -55,8 +56,7 @@ static status   statusFrame(FrameObj fr, Name stat);
 
 static status
 initialiseFrame(FrameObj fr, Name label, Name kind,
-		DisplayObj display,
-		Application app)
+		DisplayObj display, Application app)
 { if ( isDefault(kind) )
     kind = NAME_toplevel;
   if ( isDefault(display) )
@@ -135,6 +135,22 @@ getConvertFrame(Class class, PceWindow sw)
 { answer(getFrameWindow(sw, DEFAULT));
 }
 
+static status
+displayFrame(FrameObj fr, DisplayObj d)
+{ if ( fr->display != d )
+  { DisplayObj odsp = fr->display;
+
+    appendChain(d->frames, fr);
+    assign(fr, display, d);
+    deleteChain(odsp->frames, fr);
+    if ( emptyChain(odsp->frames) && isOn(odsp->removed) )
+      send(odsp, NAME_removed, EAV);
+  }
+
+  succeed;
+}
+
+
 		 /*******************************
 		 *	     SAVE-LOAD		*
 		 *******************************/
@@ -185,15 +201,14 @@ initialiseNewSlotFrame(FrameObj fr, Variable var)
 static Constant ConstantNotReturned;
 
 Any
-getConfirmFrame(FrameObj fr, Point pos, BoolObj grab, BoolObj normalise)
+getConfirmFrame(FrameObj fr, Point pos, BoolObj grab)
 { Any rval;
 
-  TRY( openFrame(fr, pos, grab, normalise) &&
+  TRY( openFrame(fr, pos, DEFAULT, grab) &&
        exposeFrame(fr) );
   busyCursorDisplay(fr->display, NIL, DEFAULT);
 
   assign(fr, return_value, ConstantNotReturned);
-  synchroniseDisplay(fr->display);
   while( offFlag(fr, F_FREED|F_FREEING) &&
 	 fr->return_value == ConstantNotReturned )
   { dispatchDisplay(fr->display);
@@ -217,7 +232,7 @@ getConfirmFrame(FrameObj fr, Point pos, BoolObj grab, BoolObj normalise)
 
 
 Any
-getConfirmCenteredFrame(FrameObj fr, Any where, BoolObj grab, Monitor mon)
+getConfirmCenteredFrame(FrameObj fr, Any where, BoolObj grab, DisplayObj dsp)
 { TRY( send(fr, NAME_create, EAV) );
 
   if ( isDefault(where) && notNil(fr->transient_for) )
@@ -235,7 +250,7 @@ getConfirmCenteredFrame(FrameObj fr, Any where, BoolObj grab, Monitor mon)
     send(fr, NAME_transientFor, rfr, EAV);
   }
 
-  return getConfirmFrame(fr, DEFAULT, grab, OFF);
+  return getConfirmFrame(fr, DEFAULT, grab);
 }
 
 
@@ -248,70 +263,28 @@ returnFrame(FrameObj fr, Any obj)
 
 
 static status
-openFrame(FrameObj fr, Point pos, BoolObj grab, BoolObj normalise)
+openFrame(FrameObj fr, Point pos, DisplayObj dsp, BoolObj grab)
 { Int x, y;
   Int w = DEFAULT, h = DEFAULT;
 
-  if ( !createdFrame(fr) )
-    TRY( send(fr, NAME_create, EAV) );
-
   if ( isDefault(pos) && isOpenFrameStatus(fr->status) )
     succeed;
   if ( notDefault(pos) )
     assign(fr, placed, ON);
 
-  if ( notDefault(pos) )		/* X11 transient is done by WM */
+  if ( notDefault(pos) )
   { x = pos->x;
     y = pos->y;
 
-#ifdef WIN32_GRAPHICS
-  setpos:
-#endif
-    if ( normalise == ON )
-    { int mx, my, mw, mh;
-      int fw = valInt(fr->area->w), fh = valInt(fr->area->h);
-      Area a;
-      Area tmp = tempObject(ClassArea,
-			    x, y, fr->area->w, fr->area->h, EAV);
-      Monitor mon = getMonitorDisplay(fr->display, tmp);
-
-      considerPreserveObject(tmp);
-
-      if ( !mon )
-	mon = getMonitorDisplay(fr->display, DEFAULT);
-      if ( !mon )
-	mon = getHeadChain(fr->display->monitors);
-      a = isNil(mon->work_area) ? mon->area : mon->work_area;
-
-      mx = valInt(a->x);
-      my = valInt(a->y);
-      mw = valInt(a->w);
-      mh = valInt(a->h);
-
-      if ( valInt(x) + fw > mx+mw ) x = toInt(mx+mw - fw);
-      if ( valInt(y) + fh > my+mh ) y = toInt(my+mh - fh);
-      if ( valInt(x) < mx ) x = toInt(mx);
-      if ( valInt(y) < my ) y = toInt(my);
-    }
-
     setFrame(fr, x, y, w, h, DEFAULT);
   } else if ( notNil(fr->geometry) )
   { ws_x_geometry_frame(fr, fr->geometry, DEFAULT);
   }
-#ifdef WIN32_GRAPHICS			/* But in Windows `do-it-yourself' */
-  else if ( notNil(fr->transient_for) )
-  { Area pa = fr->transient_for->area;
-    int xb, yb, ycap;
 
-    ws_frame_border(fr, &xb, &yb, &ycap);
-
-    x = toInt(valInt(pa->x) + xb*2);
-    y = toInt(valInt(pa->y) + yb*2 + ycap);
-    if ( isDefault(normalise) )
-      normalise = ON;
-    goto setpos;
+  if ( !createdFrame(fr) )
+  { DEBUG(NAME_frame, Cprintf("Creating %s\n", pp(fr)));
+    TRY( send(fr, NAME_create, EAV) );
   }
-#endif
 
   if ( !isOpenFrameStatus(fr->status) )
     return sdl_send(fr, NAME_status, false, NAME_window, EAV);
@@ -321,17 +294,17 @@ openFrame(FrameObj fr, Point pos, BoolObj grab, BoolObj normalise)
 
 
 static status
-openCenteredFrame(FrameObj fr, Point pos, BoolObj grab, Monitor mon)
+openCenteredFrame(FrameObj fr, Point pos, DisplayObj dsp, BoolObj grab)
 { int x, y;
   int rval;
   Point p2;
 
   TRY( send(fr, NAME_create, EAV) );
 
-  get_position_from_center_frame(fr, mon, pos, &x, &y);
+  get_position_from_center_frame(fr, dsp, pos, &x, &y);
   ensure_on_display(fr, DEFAULT, &x, &y);
   p2 = answerObject(ClassPoint, toInt(x), toInt(y), EAV);
-  rval = openFrame(fr, p2, grab, OFF);
+  rval = openFrame(fr, p2, dsp, grab);
   doneObject(p2);
 
   return rval;
@@ -526,9 +499,7 @@ statusFrame(FrameObj fr, Name stat)
     assign(fr, status, stat);
 
     if ( opened )
-    { resizeFrame(fr);
-      flushFrame(fr);
-    }
+      resizeFrame(fr);
   }
 
   succeed;
@@ -594,13 +565,19 @@ getShowFrame(FrameObj fr)
 
 status
 exposeFrame(FrameObj fr)
+{ sdl_send(fr, NAME_SdlExpose, false, EAV);
+
+  succeed;
+}
+
+static status
+SdlExposeFrame(FrameObj fr)
 { showFrame(fr, ON);
   ws_raise_frame(fr);
 
   succeed;
 }
 
-
 status
 hideFrame(FrameObj fr)
 { ws_lower_frame(fr);
@@ -627,57 +604,25 @@ hiddenFrame(FrameObj fr)
 }
 
 		/********************************
-		*       AREA MANAGEMENENT	*
+		*       AREA MANAGENENT		*
 		********************************/
 
-static Monitor
-CurrentMonitor(FrameObj fr)
-{ DisplayObj d = fr->display;
-  Monitor mon;
-
-  if ( isOpenFrameStatus(fr->status) )
-  { if ( notNil(d) && (mon=getMonitorDisplay(d, fr->area)) )
-      return mon;
-  } else if ( notNil(d) && instanceOfObject(EVENT->value, ClassEvent) )
-  { EventObj ev = EVENT->value;
-    Point pt;
-
-    if ( notNil(ev->window) && offFlag(ev->window, F_FREEING|F_FREED) )
-    { pt = getPositionEvent(ev, d);
-    } else
-    { pt = getPointerLocationDisplay(d);
-    }
-
-    if ( pt && (mon=getMonitorDisplay(d, pt)) )
-      return mon;
-  }
-
-  if ( isNil(d) )
-    d = CurrentDisplay(fr);
-
-  if ( notNil(d->monitors) )
-    return getHeadChain(d->monitors);
-
-  return NULL;
-}
-
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Get XY coordinate of frame if  its  center   must  be  at pos. If Pos is
-DEFAULT it is centered in the given   monitor. If the monitor is default
+DEFAULT it is centered in the given   display. If the monitor is default
 too, we deduce the most recent monitor from the event.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
-get_position_from_center_frame(FrameObj fr, Monitor mon, Point pos,
+get_position_from_center_frame(FrameObj fr, DisplayObj d, Point pos,
 			       int *x, int *y)
 { if ( isDefault(pos) )
-  { if ( isDefault(mon) )
-      mon = CurrentMonitor(fr);
+  { d = CurrentDisplay(fr);
 
-    if ( mon )
-    { *x = valInt(mon->area->x) + valInt(mon->area->w)/2;
-      *y = valInt(mon->area->y) + valInt(mon->area->h)/2;
+    if ( d )
+    { Area a = d->work_area;
+      *x = valInt(a->x) + valInt(a->w)/2;
+      *y = valInt(a->y) + valInt(a->h)/2;
     } else
     { *x = *y = 0;
     }
@@ -694,23 +639,24 @@ get_position_from_center_frame(FrameObj fr, Monitor mon, Point pos,
 
 
 static void
-ensure_on_display(FrameObj fr, Monitor mon, int *x, int *y)
+ensure_on_display(FrameObj fr, DisplayObj dsp, int *x, int *y)
 { int rm, bm;
 
-  if ( isDefault(mon) )
-    mon = CurrentMonitor(fr);
+  if ( isDefault(dsp) )
+    dsp = CurrentDisplay(fr);
 
-  rm = valInt(mon->area->x) + valInt(mon->area->w);
-  bm = valInt(mon->area->y) + valInt(mon->area->h);
+  Area a = dsp->work_area;
+  rm = valInt(a->x) + valInt(a->w);
+  bm = valInt(a->y) + valInt(a->h);
 
   if ( *x + valInt(fr->area->w) > rm )
     *x -= *x + valInt(fr->area->w) - rm;
   if ( *y + valInt(fr->area->h) > bm )
     *y -= *y + valInt(fr->area->h) - bm;
-  if ( *x < valInt(mon->area->x) )
-    *x = valInt(mon->area->x);
-  if ( *y < valInt(mon->area->y) )
-    *y = valInt(mon->area->y);
+  if ( *x < valInt(dsp->area->x) )
+    *x = valInt(dsp->area->x);
+  if ( *y < valInt(dsp->area->y) )
+    *y = valInt(dsp->area->y);
 }
 
 
@@ -740,16 +686,6 @@ getBoundingBoxFrame(FrameObj fr)
 
 
 
-static Monitor
-getMonitorFrame(FrameObj fr)
-{ if ( notNil(fr->display) )
-    answer(getMonitorDisplay(fr->display, fr->area));
-
-  fail;
-}
-
-
-
 static Name
 getGeometryFrame(FrameObj fr)
 { int x, y, ww, wh;
@@ -758,29 +694,15 @@ getGeometryFrame(FrameObj fr)
   { int mx, my, mw, mh;
     int xn=FALSE, yn=FALSE;
     char buf[100];
-    Monitor mon;
+    DisplayObj d = fr->display;
     int cw, ch;
 
     cw = valInt(fr->area->w);			/* Client area */
     ch = valInt(fr->area->h);
 
-    if ( (mon=getMonitorFrame(fr)) )
-    { Area a = (isNil(mon->work_area) ? mon->area : mon->work_area);
-
-      mx = valInt(a->x);
-      my = valInt(a->y);
-      mw = valInt(a->w);
-      mh = valInt(a->h);
-
-      DEBUG(NAME_geometry, Cprintf("%s on %s: %d %d %d %d\n", pp(fr), pp(mon),
-				   mx, my, mw, mh));
-    } else
-    { Size size = getSizeDisplay(fr->display);
-
-      mx = my = 0;
-      mw = valInt(size->w);
-      mh = valInt(size->h);
-    }
+    mx = my = 0;
+    mw = valInt(d->area->w);
+    mh = valInt(d->area->h);
 
     if ( x-mx > ((mx+mw) - (x+ww))*2 )	/* over 2/3th */
     { x = (mx+mw) - (x+ww);
@@ -803,12 +725,10 @@ getGeometryFrame(FrameObj fr)
     sprintf(buf+strlen(buf),
 	    "%s%d%s%d", xn ? "-" : "+", x, yn ? "-" : "+", y);
 
-    if ( mon && fr->display->monitors->size != ONE )
-    { Int n = getIndexChain(fr->display->monitors, mon);
+    Int n = getIndexChain(d->display_manager->members, d);
 
-      if ( n )
-	sprintf(buf+strlen(buf), "@%" PRIdPTR, valInt(n)-1);
-    }
+    if ( n != ONE )
+      sprintf(buf+strlen(buf), "@%" PRIdPTR, valInt(n)-1);
 
     answer(CtoName(buf));
   }
@@ -828,49 +748,47 @@ getImageFrame(FrameObj fr)
 
 
 static status
-geometryFrame(FrameObj fr, Name spec, Monitor mon)
+geometryFrame(FrameObj fr, Name spec, DisplayObj dsp)
 { assign(fr, geometry, spec);
   assign(fr, placed, ON);
 
-  ws_x_geometry_frame(fr, spec, mon);
+  ws_x_geometry_frame(fr, spec, dsp);
 
   succeed;
 }
 
 
 static status
-setFrame(FrameObj fr, Int x, Int y, Int w, Int h, Monitor mon)
+setFrame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj dsp)
 { Area a = fr->area;
-  Int ow = a->w;
-  Int oh = a->h;
 
-  if ( notDefault(mon) )
+  if ( notDefault(dsp) )
   { if ( notDefault(x) )
-      x = add(x, mon->area->x);
+      x = add(x, dsp->work_area->x);
     if ( notDefault(y) )
-      y = add(y, mon->area->y);
-
-    mon = DEFAULT;
+      y = add(y, dsp->work_area->y);
   }
 
-  if ( notDefault(x) || notDefault(y) )
-    assign(fr, placed, ON);
-  setArea(a, x, y, w, h);
-  if ( valInt(a->w) <= 0 )		/* Window systems don't like that */
-    assign(a, w, ONE);
-  if ( valInt(a->h) <= 0 )
-    assign(a, h, ONE);
-
   if ( createdFrame(fr) )
-  { ws_geometry_frame(fr, x, y, w, h, mon);
-
-    if ( ow != a->w || oh != a->h )
-      resizeFrame(fr);
+  { setArea(a, x, y, DEFAULT, DEFAULT);
+    sdl_send(fr, NAME_SdlSet, false, x, y, w, h, dsp, EAV);
+  } else
+  { if ( notDefault(x) || notDefault(y) )
+      assign(fr, placed, ON);
+    setArea(a, x, y, w, h);
+    if ( valInt(a->w) <= 0 )		/* Window systems don't like that */
+      assign(a, w, ONE);
+    if ( valInt(a->h) <= 0 )
+      assign(a, h, ONE);
   }
 
   succeed;
 }
 
+static status
+SdlSetFrame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj dsp)
+{ return ws_geometry_frame(fr, x, y, w, h, dsp);
+}
 
 static status
 xFrame(FrameObj fr, Int x)
@@ -909,10 +827,10 @@ positionFrame(FrameObj fr, Point pos)
 
 
 static status
-centerFrame(FrameObj fr, Point pos, Monitor mon)
+centerFrame(FrameObj fr, Point pos, DisplayObj dsp)
 { int x, y;
 
-  get_position_from_center_frame(fr, mon, pos, &x, &y);
+  get_position_from_center_frame(fr, dsp, pos, &x, &y);
   return setFrame(fr, toInt(x), toInt(y), DEFAULT, DEFAULT, DEFAULT);
 }
 
@@ -1051,18 +969,6 @@ getClosedFrame(FrameObj fr)
 		*          MISCELENEOUS		*
 		********************************/
 
-static status
-flushFrame(FrameObj fr)
-{ return flushDisplay(fr->display);
-}
-
-
-static status
-synchroniseFrame(FrameObj fr)
-{ return synchroniseDisplay(fr->display);
-}
-
-
 static status
 bellFrame(FrameObj fr, Int volume)
 { return bellDisplay(fr->display, volume);
@@ -1737,7 +1643,7 @@ getThreadFrame(FrameObj fr)
 /* Type declarations */
 
 static char *T_openCentered[] =
-        { "center=[point|frame]", "grab=[bool]", "monitor=[monitor]" };
+        { "center=[point|frame]", "display=[display]", "grab=[bool]" };
 static char *T_busyCursor[] =
         { "cursor=[cursor]*", "block_input=[bool]" };
 static char *T_icon[] =
@@ -1751,21 +1657,21 @@ static char *T_label[] =
         { "label=name", "icon_label=[name]" };
 static char *T_postscript[] =
         { "landscape=[bool]", "scale_in=[area]" };
-static char *T_positionADpointD_grabADboolD_normaliseADboolD[] =
-        { "position=[point]", "grab=[bool]", "normalise=[bool]" };
+static char *T_open[] =
+        { "position=[point]", "display=[display]", "grab=[bool]" };
 static char *T_wmProtocol[] =
         { "protocol=name", "action=code" };
 static char *T_convertOldSlot[] =
         { "slot=name", "value=any" };
 static char *T_set[] =
         { "x=[int]", "y=[int]", "width=[int]", "height=[int]",
-	  "monitor=[monitor]" };
+	  "display=[display]" };
 static char *T_grab_pointer[] =
 	{ "grab=bool", "cursor=[cursor]" };
 static char *T_center[] =
-	{ "center=[point]", "monitor=[monitor]" };
+	{ "center=[point]", "display=[display]" };
 static char *T_geometry[] =
-	{ "geometry=name", "monitor=[monitor]" };
+	{ "geometry=name", "display=[display]" };
 
 /* Instance Variables */
 
@@ -1835,6 +1741,8 @@ static senddecl send_frame[] =
      DEFAULT, "Convert old `show' slot"),
   SM(NAME_initialise, 4, T_initialise, initialiseFrame,
      DEFAULT, "Create from label, kind and display"),
+  SM(NAME_display, 1, "display", displayFrame,
+     NAME_organisation, "The display of the frame has changed"),
   SM(NAME_initialiseNewSlot, 1, "var=variable", initialiseNewSlotFrame,
      DEFAULT, "Initialise <-background"),
   SM(NAME_reset, 0, NULL, resetFrame,
@@ -1845,10 +1753,6 @@ static senddecl send_frame[] =
      statusFrame, DEFAULT, "Current visibility of the frame"),
   SM(NAME_typed, 1, "event|event_id", typedFrame,
      NAME_accelerator, "Dispatch over available windows"),
-  SM(NAME_flush, 0, NULL, flushFrame,
-     NAME_animate, "Flush X-server"),
-  SM(NAME_synchronise, 0, NULL, synchroniseFrame,
-     NAME_animate, "->flush and process pending events"),
   SM(NAME_border, 1, "thickness=int", borderFrame,
      NAME_appearance, "X-border width"),
   SM(NAME_showLabel, 1, "show=bool", showLabelFrame,
@@ -1863,6 +1767,8 @@ static senddecl send_frame[] =
      NAME_area, "Move the frame on the display"),
   SM(NAME_set, 5, T_set, setFrame,
      NAME_area, "Set XYWH of frame on display"),
+  SM(NAME_SdlSet, 5, T_set, SdlSetFrame,
+     NAME_area, "Update open frame position and size"),
   SM(NAME_size, 1, "size=size", sizeFrame,
      NAME_area, "Resize the frame"),
   SM(NAME_width, 1, "width=int", widthFrame,
@@ -1897,7 +1803,7 @@ static senddecl send_frame[] =
      NAME_open, "Establish window-system counterpart (internal)"),
   SM(NAME_mapped, 1, "bool", mappedFrame,
      NAME_open, "Inform transients using ->show"),
-  SM(NAME_open, 3, T_positionADpointD_grabADboolD_normaliseADboolD, openFrame,
+  SM(NAME_open, 3, T_open, openFrame,
      NAME_open, "->create and map on the display"),
   SM(NAME_openCentered, 3, T_openCentered, openCenteredFrame,
      NAME_open, "Open centered around point"),
@@ -1917,6 +1823,8 @@ static senddecl send_frame[] =
      NAME_report, "Report message (send to <-members)"),
   SM(NAME_expose, 0, NULL, exposeFrame,
      NAME_stacking, "Put frame above all others on the display"),
+  SM(NAME_SdlExpose, 0, NULL, SdlExposeFrame,
+     NAME_stacking, "SDL Thread support message"),
   SM(NAME_exposed, 0, NULL, exposedFrame,
      NAME_stacking, "Inform transient windows to expose"),
   SM(NAME_hidden, 0, NULL, hiddenFrame,
@@ -1970,8 +1878,6 @@ static getdecl get_frame[] =
      NAME_area, "Position on the display"),
   GM(NAME_size, 0, "size", NULL, getSizeFrame,
      NAME_area, "Size on the display"),
-  GM(NAME_monitor, 0, "monitor", NULL, getMonitorFrame,
-     NAME_organisation, "Monitor frame is displayed on"),
   GM(NAME_image, 1, "image", "[{bitmap,pixmap}]", getImageFrame,
      NAME_conversion, "Image with the pixels of the frame"),
   GM(NAME_keyboardFocus, 0, "window", NULL, getKeyboardFocusFrame,
@@ -1984,8 +1890,7 @@ static getdecl get_frame[] =
      NAME_icon, "(Current) position of the icon"),
   GM(NAME_tile, 0, "tile", NULL, getTileFrame,
      NAME_layout, "Find tile managing object"),
-  GM(NAME_confirm, 3, "return_value=any",
-     T_positionADpointD_grabADboolD_normaliseADboolD, getConfirmFrame,
+  GM(NAME_confirm, 2, "return_value=any", T_open, getConfirmFrame,
      NAME_modal, "Start sub-eventloop until ->return"),
   GM(NAME_confirmCentered, 3, "return_value=any",
      T_openCentered, getConfirmCenteredFrame,
@@ -2032,9 +1937,7 @@ static classvardecl rc_frame[] =
   RC(NAME_fitAfterAppend, "bool", "@off",
      "Automatically ->fit the frame after a subwindow was added"),
   RC(NAME_decorateTransient, "bool", "@on",
-     "Decorate transient windows (if possible)"),
-  RC(NAME_colourMap, "[colour_map]*", "@default",
-     "Colourmap for the window's frame")
+     "Decorate transient windows (if possible)")
 };
 
 /* Class Declaration */
