@@ -194,21 +194,18 @@ unlinkWindow(PceWindow sw)
 		********************************/
 
 static status
-openWindow(PceWindow sw, Point pos, BoolObj normalise)
-{ if ( send(sw, NAME_create, EAV) &&
-       send(getFrameWindow(sw, DEFAULT), NAME_open,
-	    pos, DEFAULT, normalise, EAV) )
-  succeed;
-
-  fail;
+openWindow(PceWindow sw, Point pos, DisplayObj dsp)
+{ return ( send(sw, NAME_create, EAV) &&
+	   send(getFrameWindow(sw, DEFAULT), NAME_open,
+		pos, EAV) );
 }
 
 
 static status
-openCenteredWindow(PceWindow sw, Point pos, BoolObj grab, Monitor mon)
+openCenteredWindow(PceWindow sw, Point pos, DisplayObj dsp, BoolObj grab)
 { if ( send(sw, NAME_create, EAV) &&
        send(getFrameWindow(sw, DEFAULT), NAME_openCentered,
-	    pos, grab, mon, EAV) )
+	    pos, dsp, grab, EAV) )
     succeed;
 
   fail;
@@ -216,19 +213,19 @@ openCenteredWindow(PceWindow sw, Point pos, BoolObj grab, Monitor mon)
 
 
 static Any
-getConfirmWindow(PceWindow sw, Any pos, BoolObj grab, BoolObj normalise)
+getConfirmWindow(PceWindow sw, Any pos, BoolObj grab)
 { TRY( send(sw, NAME_create, EAV) );
 
-  answer(getConfirmFrame(getFrameWindow(sw, DEFAULT), pos, grab, normalise));
+  answer(getConfirmFrame(getFrameWindow(sw, DEFAULT), pos, grab));
 }
 
 
 static Any
-getConfirmCenteredWindow(PceWindow sw, Any pos, BoolObj grab, Monitor mon)
+getConfirmCenteredWindow(PceWindow sw, Any pos, DisplayObj dsp, BoolObj grab)
 { TRY( send(sw, NAME_create, EAV) );
 
   answer(getConfirmCenteredFrame(getFrameWindow(sw, DEFAULT),
-				 pos, grab, mon));
+				 pos, grab, dsp));
 }
 
 
@@ -492,31 +489,6 @@ resizeMessageWindow(PceWindow sw, Code msg)
 }
 
 
-static Monitor
-getMonitorWindow(PceWindow sw)
-{ if ( isNil(sw->device) )
-  { DisplayObj d = getDisplayGraphical((Graphical)sw);
-
-    if ( d )
-    { FrameObj fr;
-      int dx, dy;
-      struct area a;
-
-      frame_offset_window(sw, &fr, &dx, &dy);
-      a = *fr->area;
-      a.x = toInt(valInt(a.x)+dx);
-      a.y = toInt(valInt(a.y)+dy);
-
-      answer(getMonitorDisplay(d, &a));
-    }
-
-    fail;
-  }
-
-  return getMonitorGraphical((Graphical)sw);
-}
-
-
 		/********************************
 		*           COMPUTING		*
 		********************************/
@@ -2036,9 +2008,7 @@ flushWindow(PceWindow sw)
 { DisplayObj d = getDisplayGraphical((Graphical) sw);
 
   if ( d )
-  { pceRedrawWindow(sw);
-    ws_flush_display(d);
-  }
+    pceRedrawWindow(sw);
 
   succeed;
 }
@@ -2177,7 +2147,7 @@ getThreadWindow(PceWindow sw)
 /* Type declarations */
 
 static char *T_open[] =
-        { "[point]", "normalise=[bool]" };
+        { "[point]", "display=[display]" };
 static char *T_scrollHV[] =
         { "direction={forwards,backwards,goto}",
 	  "unit={page,file,line}",
@@ -2187,7 +2157,7 @@ static char *T_scrollHV[] =
 static char *T_decorate[] =
         { "area=[{grow,shrink}]", "left_margin=[int]", "right_margin=[int]", "top_margin=[int]", "bottom_margin=[int]", "decorator=[window]" };
 static char *T_confirmCentered[] =
-        { "center=[point|frame]", "grab=[bool]", "monitor=[monitor]" };
+        { "center=[point|frame]", "display=[display]", "grab=[bool]" };
 static char *T_typed[] =
         { "event|event_id", "delegate=[bool]" };
 static char *T_focus[] =
@@ -2199,7 +2169,7 @@ static char *T_catchAll[] =
 static char *T_changedUnion[] =
         { "ox=int", "oy=int", "ow=int", "oh=int" };
 static char *T_confirm[] =
-        { "position=[point]", "grab=[bool]", "normalise=[bool]" };
+        { "position=[point]", "grab=[bool]" };
 static char *T_geometry[] =
         { "x=[int]", "y=[int]", "width=[int]", "height=[int]" };
 static char *T_flash[] =
@@ -2396,11 +2366,9 @@ static getdecl get_window[] =
      NAME_area, "New area representing visible part"),
   GM(NAME_size, 0, "size", NULL, getSizeGraphical,
      NAME_area, "New size representing size (avoid class-variable)"),
-  GM(NAME_monitor, 0, "monitor", NULL, getMonitorWindow,
-     NAME_organisation, "Monitor window is displayed on"),
   GM(NAME_displayedCursor, 0, "cursor*", NULL, getDisplayedCursorDevice,
      NAME_cursor, "Currently displayed cursor"),
-  GM(NAME_confirm, 3, "any", T_confirm, getConfirmWindow,
+  GM(NAME_confirm, 2, "any", T_confirm, getConfirmWindow,
      NAME_modal, "Run sub event-loop until ->return"),
 #ifdef WIN32_GRAPHICS
   GM(NAME_winHandle, 0, "int", NULL, getWinHandleWindow,
@@ -2419,8 +2387,8 @@ static classvardecl rc_window[] =
 { RC(NAME_background,        "colour|pixmap",  UXWIN("white", "@_graph_bg"), NULL),
   RC(NAME_cursor,            "cursor",        "arrow",                       NULL),
   RC(NAME_pen,               "0..",	      "@_win_pen",                   NULL),
-  RC(NAME_selectionHandles,  RC_REFINE,	      "@nil",	                 NULL),
-  RC(NAME_size,	         "size",	      "size(200,100)",               NULL),
+  RC(NAME_selectionHandles,  RC_REFINE,	      "@nil",			 NULL),
+  RC(NAME_size,		 "size",	      "size(200,100)",               NULL),
   RC(NAME_selectionFeedback, NULL,            "colour",                      NULL)
 };
 
