@@ -604,59 +604,6 @@ initHostConstants(void)
 					       "pce_principal");
 }
 
-
-#ifdef __WINDOWS__
-#include <console.h>
-
-static RlcUpdateHook old_update_hook;
-
-static void *
-getConsoleFunction(const char *name)
-{ HMODULE hconsole;
-
-  if ( (hconsole=GetModuleHandle("plterm")) )
-  { return GetProcAddress(hconsole, name);
-  }
-
-  return NULL;
-}
-
-
-RlcUpdateHook
-indirect_rlc_update_hook(RlcUpdateHook hook)
-{ RlcUpdateHook (*sethook)(RlcUpdateHook new);
-
-  if ( (sethook = getConsoleFunction("rlc_update_hook")) )
-    return (*sethook)(hook);
-
-  return NULL;
-}
-
-
-static HWND
-indirect_rlc_hwnd()
-{ HWND (*f)(void *console);
-
-  if ( (f = getConsoleFunction("rlc_hwnd")) )
-    return (*f)(NULL);
-
-  return 0;
-}
-
-
-#define PROLOG_ITF_INIT() \
-	{ }
-#define PROLOG_INSTALL_REDRAW_FUNCTION(f) \
-	{ old_update_hook = indirect_rlc_update_hook(f); }
-#ifndef O_SHAREDLIBRARY
-#define O_SHAREDLIBRARY
-#endif
-#endif
-
-#ifdef O_SHAREDLIBRARY
-#ifndef PROLOG_ITF_INIT
-#define PROLOG_ITF_INIT() { }
-#endif
 #define PROLOG_ONEXIT(f)	{ exitpce_hook = (OnExitFunction) f; }
 
 static OnExitFunction		exitpce_hook;
@@ -694,9 +641,7 @@ install_pl2xpce(void)
   PL_register_foreign("pce_open_terminal_image", 4,
 		      pl_pce_open_terminal_image, 0);
 
-#ifndef __WINDOWS__
   PL_license("lgplv2+", "xpce (pango library)");
-#endif
 
   install_pcecall();
 }
@@ -711,16 +656,11 @@ uninstall_pl2xpce(void)
 		 old_dispatch_hook, old_update_hook));
 
   PL_dispatch_hook(old_dispatch_hook);
-#ifdef __WINDOWS__
-  indirect_rlc_update_hook(old_update_hook);
-#endif
   if ( exitpce_hook )
     (*exitpce_hook)(0);
 }
 
-#endif /*O_SHAREDLIBRARY*/
-
-#endif /*SWI*/
+#endif/*SWI*/
 
 		 /*******************************
 		 *         EXCEPTIONS		*
@@ -2139,9 +2079,11 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 	  else
 	    prof_node = NULL;
 	  if ( ret )
-	  { rval = PL_call_predicate(MODULE_user,
+	  { int locks = pceMTUnlockAll();
+	    rval = PL_call_predicate(MODULE_user,
 				     DebugMode|PL_Q_PASS_EXCEPTION,
 				     PREDICATE_get_implementation, av);
+	    pceMTRelock(locks);
 	    if ( rval )
 	    { if ( IsFunctor(av+3, FUNCTOR_ref1) )
 	      { if ( !get_object_from_refterm(av+3, &goal.rval) )
@@ -2155,9 +2097,11 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 		rval = PL_unify(ret, av+3);
 	    }
 	  } else
-	  { rval = PL_call_predicate(MODULE_user,
+	  { int locks = pceMTUnlockAll();
+	    rval = PL_call_predicate(MODULE_user,
 				     DebugMode|PL_Q_PASS_EXCEPTION,
 				     PREDICATE_send_implementation, av);
+	    pceMTRelock(locks);
 	  }
 	  if ( prof_node )
 	    PL_prof_exit(prof_node);
@@ -2853,7 +2797,7 @@ pl_pce_dispatch_event(term_t Fd, term_t timeout)
     return false;
 
   pceDispatch(fd, tmo*1000.0);
-  return true;
+  return !PL_exception(0);
 }
 
 static foreign_t
@@ -3070,10 +3014,36 @@ PrologQuery(int what, PceCValue *value)
 
 static IOSTREAM *
 XPCE_OUTPUT(void)
-{ IOSTREAM *s = Suser_output;
+{ if ( PL_thread_self() > 0 )
+  { static predicate_t pred = NULL;
+
+    if ( !pred )
+      pred = PL_predicate("xpce_console", 3, "pce");
+
+    fid_t fid = PL_open_foreign_frame();
+    IOSTREAM *s = NULL;
+    if ( fid )
+    { term_t av = PL_new_term_refs(3);
+      if ( PL_call_predicate(NULL, PL_Q_NODEBUG, pred, av) )
+      { IOSTREAM *tmp;
+
+	if ( PL_get_stream(av+1, &tmp, SIO_OUTPUT) )
+	  s = tmp;
+	else
+	  Sdprintf("Could not extract stdout from pce:xpce_console/3\n");
+      }
+      PL_close_foreign_frame(fid);
+    }
+
+    if ( s )
+      return s;
+  }
+
+
+  IOSTREAM *s = Suser_output;
   if ( !s )
     s = Soutput;
-  return s;
+  return PL_acquire_stream(s);
 }
 
 static IOSTREAM *
@@ -3087,19 +3057,26 @@ XPCE_INPUT(void)
 
 void
 pl_Cvprintf(const char *fmt, va_list args)
-{ Svfprintf(XPCE_OUTPUT(), fmt, args);
+{ IOSTREAM *s = XPCE_OUTPUT();
+  Svfprintf(s, fmt, args);
+  PL_release_stream(s);
 }
 
 
 static int
 pl_Cputchar(int c)
-{ return Sputcode(c, XPCE_OUTPUT());
+{ IOSTREAM *s = XPCE_OUTPUT();
+  int rc = Sputcode(c, s);
+  PL_release_stream(s);
+  return rc;
 }
 
 
 static void
 pl_Cflush(void)
-{ Sflush(XPCE_OUTPUT());
+{ IOSTREAM *s = XPCE_OUTPUT();
+  Sflush(s);
+  PL_release_stream(s);
 }
 
 
@@ -3346,9 +3323,6 @@ static pce_callback_functions callbackfunction =
 #ifndef PROLOG_INSTALL_REINIT_FUNCTION
 #define PROLOG_INSTALL_REINIT_FUNCTION(x)
 #endif
-#ifndef PROLOG_ITF_INIT
-#define PROLOG_ITF_INIT()
-#endif
 #ifndef PROLOG_INSTALL_CALLBACKS
 #define PROLOG_INSTALL_CALLBACKS()
 #endif
@@ -3358,10 +3332,6 @@ static pce_callback_functions callbackfunction =
 #ifndef PROLOG_INSTALL_RESET_FUNCTION
 #define PROLOG_INSTALL_RESET_FUNCTION(f)
 #endif
-#ifndef PROLOG_INSTALL_REDRAW_FUNCTION
-#define PROLOG_INSTALL_REDRAW_FUNCTION(f)
-#endif
-
 
 static void
 do_reset(void)
@@ -3370,13 +3340,6 @@ do_reset(void)
 }
 
 
-#ifdef __WINDOWS__
-static void
-do_redraw(void)
-{ pceRedraw(FALSE);			/* FALSE: do not sync Xserver */
-}
-#endif
-
 static int
 hasThreadsProlog()
 { predicate_t pred = PL_predicate("current_prolog_flag", 2, "user");
@@ -3434,7 +3397,6 @@ pl_pce_init(term_t Home, term_t AppDir)
     }
 
     PROLOG_INSTALL_REINIT_FUNCTION(pl_pce_init);
-    PROLOG_ITF_INIT();
 
     pceRegisterCallbacks(&callbackfunction);
     initNameAtomTable();
@@ -3450,7 +3412,6 @@ pl_pce_init(term_t Home, term_t AppDir)
     pceSend(PROLOG, NULL, cToPceName("name_reference"), 1, &plname);
     PROLOG_INSTALL_DISPATCH_FUNCTION(pce_dispatch);
     PROLOG_INSTALL_RESET_FUNCTION(do_reset);
-    PROLOG_INSTALL_REDRAW_FUNCTION(do_redraw);
   }
 
   return TRUE;
