@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2024, University of Amsterdam
+    Copyright (c)  1985-2025, University of Amsterdam
                               VU University Amsterdam
                               SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -47,6 +47,9 @@
 #define	firstClause(av, fr, def, next)	LDFUNC(firstClause, av, fr, def, next)
 #define	nextClause(chp, argv, fr, def)	LDFUNC(nextClause, chp, argv, fr, def)
 #define getIndexOfTerm(t)		LDFUNC(getIndexOfTerm, t)
+#define ci_set_flag(value, key)		LDFUNC(ci_set_flag, value, key)
+#define ci_get_flag(term, key)		LDFUNC(ci_get_flag, term, key)
+#define update_primary_index(def)	LDFUNC(update_primary_index, def)
 #endif /*USE_LD_MACROS*/
 
 #define LDFUNC_DECLARATIONS
@@ -54,8 +57,8 @@
 word		getIndexOfTerm(term_t t);
 ClauseRef	firstClause(Word argv, LocalFrame fr, Definition def,
 			    ClauseChoice next);
-ClauseRef	nextClause(ClauseChoice chp, Word argv, LocalFrame fr,
-			   Definition def);
+ClauseRef	nextClause(const ClauseChoice chp, const Word argv,
+			   const LocalFrame fr, const Definition def);
 int		addClauseToIndexes(Definition def, Clause cl,
 				   ClauseRef where);
 void		delClauseFromIndex(Definition def, Clause cl);
@@ -66,13 +69,18 @@ void		clearTriedIndexes(Definition def);
 void		unallocClauseIndexTable(ClauseIndex ci);
 void		deleteActiveClauseFromIndexes(Definition def, Clause cl);
 bool		unify_index_pattern(Procedure proc, term_t value);
-void		deleteIndexes(ClauseList cl, int isnew);
+void		deleteIndexes(Definition def, ClauseList cl, bool isnew);
 void		deleteIndexesDefinition(Definition def);
 int		checkClauseIndexSizes(Definition def, int nindexable);
 void		checkClauseIndexes(Definition def);
 void		listIndexGenerations(Definition def, gen_t gen);
 size_t		sizeofClauseIndexes(Definition def);
-void		initClauseIndexing();
+void		initClauseIndexing(void);
+bool		ci_is_flag(atom_t key);
+bool		ci_set_flag(term_t value, atom_t key);
+bool		ci_get_flag(term_t t, atom_t key);
+void		update_primary_index(Definition def);
+word		index_of_word(word w);
 
 #undef LDFUNC_DECLARATIONS
 
